const moment = require('moment');

const updateTransactions = async (bot, transactionRepo, tagRepo, accountId) => {
	const range = await getRange(transactionRepo, accountId);
	const results = range.map(month => updateTransactionsForMonth(bot, transactionRepo, tagRepo, accountId, month));
	await Promise.all(results);
}

const updateTransactionsForMonth = async (bot, transactionRepo, tagRepo, accountId, month) => {
	await transactionRepo.clearMonth(accountId, month.format('YYYY-MM'));
	const transactions = await bot.getTransactionsForMonth(accountId, month.format('YYYY-MM'));
	await transactionRepo.createTransactions(transactions.transactions.items, accountId);
	await tagTransactions(tagRepo, month.format('YYYY-MM'))

}

const updatePendingTransactions = async (bot, transactionRepo, accountId) => {
	const transactions = await bot.getPendingTransactions(accountId);
	await transactionRepo.createPendingTransactions(transactions, accountId);
}

const updateAccount = async (bot, accountRepo, accountId) => {
	const accounts = await bot.getAccounts();
	const account = accounts.find(acc => acc.id === accountId);
	return await accountRepo.updateAccount(account);
}

const getRange = async (transactionRepo, accountId) => {
	const lastTransactionDate = await transactionRepo.getLastTransactionDate(accountId);
	const to = moment().startOf('month');
	const from = moment(lastTransactionDate).startOf('month');
	return Array.from(moment.range(from, to).by('month'));
}

const tagTransactions = async (tagRepo, from) => {
	const tags = await tagRepo.getAutoTags();
	const results = tags.map(tag => {
		tagRepo.tagTransactions(tag.tagName, tag.description, tag.isRecurring, tag.isTransfer, from)
	});
	await Promise.all(results);
}

const updateAccountAndTransactions = (bot, transactionRepo, tagRepo, accountRepo) => async (accountId) => {
	await bot.login();
	await updateAccount(bot, accountRepo, accountId);
	await updateTransactions(bot, transactionRepo, tagRepo, accountId);
	await updatePendingTransactions(bot, transactionRepo, accountId);
	await bot.logout();
}

module.exports = updateAccountAndTransactions;